/* deletes bond records from a given atom record.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <nrcvax.h>
#include "cdmod.h"


void *DeleteBondRecords( AtomRecord *record )
{
    int i, j;
    int n_bonds,
        new_n_bonds;

    char atom_name[ATOM_NAME_LEN],
         line[LINE_LEN],
         *ptr;

    AtomNameArray ordered_names;

    BondRecord  tmp_bond[N_BOND_RECORDS];

    InitBondRecord( tmp_bond );

    strncpy( atom_name, record->params.at_name, ATOM_NAME_LEN );
    atom_name[ATOM_NAME_LEN - 1] = '\0';

    n_bonds = record->n_bonds < N_BOND_RECORDS ?
                                record->n_bonds : N_BOND_RECORDS;

    fprintf( stdout, "List of bond records for atom %s\n", atom_name );
                      
    for( i = 0; i < n_bonds; i++ )
         fprintf( stdout, "%.8s %7.3f\n",  
                                         record->bond_record[i].name,
                                         record->bond_record[i].dist );
    fputs( "\n", stdout );

/* ask the user for the bond records he/she would like to keep. */

    fputs( "Enter atom names for the records to keep (CR=keep all): ", stdout );
    fflush( stdout );

    if( NULL == fgets(line, sizeof(line), stdin) ) {
        fprintf( stderr, "fgets() failed -- %s:%d\n", __FILE__, __LINE__ );
        exit( EXIT_FAILURE );
    }
    if( NULL != (ptr = strchr(line, '\n')) ) *ptr = '\0';

    if( 0 == strncmp(line, "", 1) ) {
        fputs( "Keeping all bond records.\n", stdout );
        return NULL;
    }

    ptr = ConvertNames( line );
    if( NULL == ptr ) {
        fprintf( stderr, "Input problem with line: %s\n", line );
        return NULL;
    }

    ordered_names = SplitAtomNameList( line );
    if( NULL == ordered_names ) {
        fprintf( stderr, "Problem parsing data in: %s\n", line );
        return NULL;
    }
  
    new_n_bonds = WordCount( line );
    if( (new_n_bonds > n_bonds) || (new_n_bonds > N_BOND_RECORDS) ) {
       fprintf( stderr, "Too many records specified (%d).\n", new_n_bonds );
       return NULL;
    }
    
        

/* write the bond records to keep in the user specified order using a 
 * temporary array, and copy back into 'record's bond_record array.
 */
    for( i = 0; i < new_n_bonds; i++ ) {
         for( j = 0; j < n_bonds; j++ ) {
              if( 0 == memcmp(ordered_names[i],
                               record->bond_record[j].name, ATOM_NAME_LEN) )
                  tmp_bond[i] = record->bond_record[j];
         }
    }

    record->n_bonds = new_n_bonds;
    memcpy( record->bond_record, tmp_bond, N_BOND_RECORDS * sizeof(BondRecord) );

    free( ordered_names );
    return record;
}
