/* this is the  main() driver function for .cd file hacking functions.
 * It seemed easier to change the .cd file rather than writing FORTRAN
 * to do the same thing.
 * Written by:
 *	Paul D. Boyle
 *	Department of Chemistry
 *	Box 8204
 *	North Carolina State University
 *	Raleigh, NC 27695-8204
 *
 *	September 1999
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <nrcvax.h>

#include "cdmod.h"


int main( int argc, char **argv )
{

    FILE *cdfile;
    int at_num = 1;  /* start at first atom */
    int yes_no;

    AtomRecord *tmp;
    struct atom_list head,
                     *current,
                     *tmp_list;

   if( argc != 2 ) {
       Usage( argv[argc-1] );
       exit( EXIT_FAILURE );
   }

    cdfile = fopen( argv[argc-1], "rb+" );
    if( !cdfile ) {
        fprintf( stderr, "Couldn't open file: %s Exiting.\n", argv[argc-1] );
        exit( EXIT_FAILURE );
    }       

/* initialize the linked list */
    current = head.next = malloc(sizeof( *current ));
    if( !current ) {
        fprintf( stderr, "malloc() failed -- %s:%d Exiting.\n", 
                          __FILE__, __LINE__ );
        exit( EXIT_FAILURE );
    }
    current->next = NULL;

/* fill the linked list with the atom records. */
    while( NULL != (tmp = ReadAtomRecord(cdfile, at_num)) ) {
        current->next = malloc( sizeof(*current) );
        if( !current->next ) {
            fprintf( stderr,
                 "malloc() failed -- %s:%d after atom number %d. Exiting.\n",
                  __FILE__, __LINE__, at_num );
            exit( EXIT_FAILURE );
        } 
        current->atom_num = at_num;
        current->atom_record = tmp;
        current = current->next;
        current->next = NULL;
        at_num++;
    }
   
/* change the bonding records */ 
    ModifyBondRecords( head.next );

    yes_no = YesNo( "Do you want to save the modified .cd file?" );

    if( yes_no == 'y' ) {
        fputs( "Writing records to .cd file...\n", stdout );
        WriteAtomRecords( cdfile, head.next );
    }

/* free the linked list */
   
    for( current = head.next; current; current = tmp_list ) {
         free( current->atom_record );
         tmp_list = current->next;
         free( current );
    }
    
    fclose( cdfile );
    fputs( "Finished.\n", stdout );
    exit( EXIT_SUCCESS );
}
